unit DBInfoFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, ExtCtrls, ComCtrls,
  Db, TinyDB, ImgList, Menus, BaseFrm;

type
  TNodeType = (ntTable, ntField, ntIndex);
  PNodeData = ^TNodeData;
  TNodeData = record
    NodeType: TNodeType;
    TableName: string;
    SelfName: string;
  end;

  TDBInfoForm = class(TBaseForm)
    DBStructTreeView: TTreeView;
    TinyTable: TTinyTable;
    ImageList: TImageList;
    TablePopupMenu: TPopupMenu;
    TableMenuOpenItem: TMenuItem;
    TableMenuDeleteItem: TMenuItem;
    TableMenuRenameItem: TMenuItem;
    TableMenuN1: TMenuItem;
    TableMenuRefreshItem: TMenuItem;
    TableMenuN2: TMenuItem;
    TableMenuNewItem: TMenuItem;
    FieldPopupMenu: TPopupMenu;
    FieldMenuRenameItem: TMenuItem;
    FieldMenuN1: TMenuItem;
    FieldMenuRefreshItem: TMenuItem;
    FieldMenuN2: TMenuItem;
    FieldMenuNewItem: TMenuItem;
    IndexPopupMenu: TPopupMenu;
    IndexMenuRenameItem: TMenuItem;
    IndexMenuN1: TMenuItem;
    IndexMenuRefreshItem: TMenuItem;
    IndexMenuN2: TMenuItem;
    IndexMenuNewItem: TMenuItem;
    IndexMenuDeleteItem: TMenuItem;
    TableMenuEmptyItem: TMenuItem;
    FieldMenuPropItem: TMenuItem;
    IndexMenuPropItem: TMenuItem;
    TableMenuDesignItem: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure DBStructTreeViewMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure TableMenuOpenItemClick(Sender: TObject);
    procedure TableMenuRenameItemClick(Sender: TObject);
    procedure TableMenuRefreshItemClick(Sender: TObject);
    procedure TableMenuNewItemClick(Sender: TObject);
    procedure TableMenuDeleteItemClick(Sender: TObject);
    procedure TableMenuEmptyItemClick(Sender: TObject);
    procedure IndexMenuDeleteItemClick(Sender: TObject);
    procedure IndexMenuRenameItemClick(Sender: TObject);
    procedure DBStructTreeViewEditing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure DBStructTreeViewEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure FieldMenuRenameItemClick(Sender: TObject);
    procedure FieldMenuPropItemClick(Sender: TObject);
    procedure IndexMenuPropItemClick(Sender: TObject);
    procedure DBStructTreeViewDblClick(Sender: TObject);
    procedure TableMenuDesignItemClick(Sender: TObject);
  private
    { Private declarations }
    FTinyDatabase: TTinyDatabase;
    FCurNodeData: TNodeData;
    FCurNode: TTreeNode;

    procedure FillDBStructToTreeView;
    procedure AddNodeData(TreeNode: TTreeNode; NodeType: TNodeType; TableName, SelfName: string);
    procedure FreeNodeData(TreeNode: TTreeNode);
    procedure ClearNodeData;
    function GetNodeData(TreeNode: TTreeNode): TNodeData;
  public
    { Public declarations }
    procedure SetData(ATinyDB: TTinyDatabase);
    procedure InvokeNewTableForm;
    procedure RefreshDBInfo;
    procedure OpenTable(TableName: string);
    procedure DeleteTable(TableName: string);
    procedure EmptyTable(TableName: string);
    procedure DesignTable(TableName: string);
    procedure LocateNodeOfTableName(TableName: string);
    procedure DeleteIndex(TableName, IndexName: string);
    procedure RenameTable(OldTableName, NewTableName: string);
    procedure RenameField(TableName, OldFieldName, NewFieldName: string);
    procedure RenameIndex(TableName, OldIndexName, NewIndexName: string);
    procedure RefreshFormOfTableName(TableName: string);
    procedure CloseFormOfTableName(TableName: string);
    procedure CloseAllTable;
    procedure ExportDBToText(FileName: string);
  end;

var
  DBInfoForm: TDBInfoForm;

procedure ShowDBInfoForm(ATinyDB: TTinyDatabase);

implementation

uses
  LangMgr, MainFrm, DBTableFrm, DBGridFrm, DBCardFrm, NewTableFrm,
  FieldPropFrm, IndexPropFrm, TableListFrm;

{$R *.DFM}

procedure ShowDBInfoForm(ATinyDB: TTinyDatabase);
begin
  if DBInfoForm = nil then
  begin
    DBInfoForm := TDBInfoForm.Create(Application);
    DBInfoForm.ManualDock(MainForm.LeftDockPanel);
  end;
  DBInfoForm.CloseAllTable;
  DBInfoForm.SetData(ATinyDB);
  DBInfoForm.Show;
end;

procedure TDBInfoForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  I: Integer;
begin
  if (HostDockSite is TPanel) then
    MainForm.ShowDockPanel(HostDockSite as TPanel, False, nil);

  MainForm.AdjustUI(False);
  for I := 0 to FTinyDatabase.TableDefs.Count - 1 do
    CloseFormOfTableName(FTinyDatabase.TableDefs[I].Name);
  ClearNodeData;
  FTinyDatabase.Close;
  Action := caFree;
  DBInfoForm := nil;
end;

procedure TDBInfoForm.SetData(ATinyDB: TTinyDatabase);
begin
  FTinyDatabase := ATinyDB;
  TinyTable.DatabaseName := FTinyDatabase.DatabaseName;
  FillDBStructToTreeView;
end;

procedure TDBInfoForm.InvokeNewTableForm;
begin
  if ShowNewTableForm then
    FillDBStructToTreeView;
end;

procedure TDBInfoForm.RefreshDBInfo;
begin
  FillDBStructToTreeView;
end;

procedure TDBInfoForm.OpenTable(TableName: string);
var
  Value: TDBTableFormData;
begin
  Value.TinyDatabase := FTinyDatabase;
  Value.TableName := TableName;
  case MainForm.OpenTableMode of
    otGrid:
      ShowDBGridForm(Value);
    otCard:
      ShowDBCardForm(Value);
  end;
  LocateNodeOfTableName(TableName);
end;

procedure TDBInfoForm.DeleteTable(TableName: string);
var
  R: Integer;
begin
  R := Application.MessageBox(PChar(AppLangMgr.Trans('Are you sure to delete?')), PChar(Application.Title), 36);
  if R = ID_YES then
  begin
    CloseFormOfTableName(TableName);
    FTinyDatabase.DeleteTable(TableName);
    FillDBStructToTreeView;
  end;
end;

procedure TDBInfoForm.EmptyTable(TableName: string);
var
  R: Integer;
begin
  R := Application.MessageBox(PChar(AppLangMgr.Trans('Are you sure to clear all records?')), PChar(Application.Title), 36);
  if R = ID_YES then
  begin
    TinyTable.TableName := TableName;
    TinyTable.Open;
    TinyTable.EmptyTable;
    TinyTable.Close;
    RefreshFormOfTableName(TableName);
  end;
end;

procedure TDBInfoForm.DesignTable(TableName: string);
begin
  CloseFormOfTableName(TableName);
  if ShowDesignTableForm(FTinyDatabase, TableName) then
    FillDBStructToTreeView;
end;

procedure TDBInfoForm.FillDBStructToTreeView;
const
  FieldsStr = 'Fields';
  IndexesStr = 'Indexes';
var
  I, J: Integer;
  TableNode, FieldNode, IndexNode, TempNode: TTreeNode;
  TableName, FieldName, IndexName: string;
begin
  TinyTable.Close;
  TinyTable.DatabaseName := FTinyDatabase.DatabaseName;
  DBStructTreeView.Items.BeginUpdate;
  ClearNodeData;
  DBStructTreeView.Items.Clear;
  for I := 0 to FTinyDatabase.TableDefs.Count - 1 do
  begin
    TableName := FTinyDatabase.TableDefs[I].Name;
    TableNode := DBStructTreeView.Items.AddChild(nil, TableName);
    TableNode.ImageIndex := 0;
    TableNode.SelectedIndex := 0;
    AddNodeData(TableNode, ntTable, TableName, TableName);

    FieldNode := DBStructTreeView.Items.AddChild(TableNode, FieldsStr);
    FieldNode.ImageIndex := 1;
    FieldNode.SelectedIndex := 1;
    FieldNode.Data := nil;

    IndexNode := DBStructTreeView.Items.AddChild(TableNode, IndexesStr);
    IndexNode.ImageIndex := 2;
    IndexNode.SelectedIndex := 2;
    IndexNode.Data := nil;

    TinyTable.TableName := TableName;
    TinyTable.Open;
    for J := 0 to TinyTable.FieldDefs.Count - 1 do
    begin
      FieldName := TinyTable.FieldDefs[J].Name;
      TempNode := DBStructTreeView.Items.AddChild(FieldNode, FieldName);
      TempNode.ImageIndex := 1;
      TempNode.SelectedIndex := 1;
      AddNodeData(TempNode, ntField, TableName, FieldName);
    end;
    for J := 0 to TinyTable.IndexDefs.Count - 1 do
    begin
      IndexName := TinyTable.IndexDefs[J].Name;
      TempNode := DBStructTreeView.Items.AddChild(IndexNode, IndexName);
      TempNode.ImageIndex := 2;
      TempNode.SelectedIndex := 2;
      AddNodeData(TempNode, ntIndex, TableName, IndexName);
    end;
    TinyTable.Close;
  end;
  DBStructTreeView.Items.EndUpdate;
  DBStructTreeView.FullExpand;
end;

procedure TDBInfoForm.AddNodeData(TreeNode: TTreeNode; NodeType: TNodeType; TableName, SelfName: string);
var
  NodeDataPtr: PNodeData;
begin
  New(NodeDataPtr);
  NodeDataPtr^.NodeType := NodeType;
  NodeDataPtr^.TableName := TableName;
  NodeDataPtr^.SelfName := SelfName;
  TreeNode.Data := NodeDataPtr;
end;

procedure TDBInfoForm.FreeNodeData(TreeNode: TTreeNode);
begin
  if TreeNode.Data = nil then Exit;
  Dispose(PNodeData(TreeNode.Data));
end;

procedure TDBInfoForm.ClearNodeData;
var
  I: Integer;
begin
  for I := 0 to DBStructTreeView.Items.Count - 1 do
    FreeNodeData(DBStructTreeView.Items[I]);
end;

function TDBInfoForm.GetNodeData(TreeNode: TTreeNode): TNodeData;
begin
  Result := PNodeData(TreeNode.Data)^;
end;

procedure TDBInfoForm.RefreshFormOfTableName(TableName: string);
var
  I: Integer;
  DBTableForm: TDBTableForm;
  DBTableForms: array of TDBTableForm;
begin
  for I := 0 to MainForm.MDIChildCount - 1 do
  begin
    if MainForm.MDIChildren[I] is TDBTableForm then
    begin
      DBTableForm := MainForm.MDIChildren[I] as TDBTableForm;
      if DBTableForm.TableName = TableName then
      begin
        SetLength(DBTableForms, Length(DBTableForms) + 1);
        DBTableForms[High(DBTableForms)] := DBTableForm;
      end;
    end;
  end;
  for I := 0 to High(DBTableForms) do
    DBTableForms[I].TinyTable.Refresh;
  SetLength(DBTableForms, 0);
end;

procedure TDBInfoForm.CloseFormOfTableName(TableName: string);
var
  I: Integer;
  DBTableForm: TDBTableForm;
  DBTableForms: array of TDBTableForm;
begin
  for I := 0 to MainForm.MDIChildCount - 1 do
  begin
    if MainForm.MDIChildren[I] is TDBTableForm then
    begin
      DBTableForm := MainForm.MDIChildren[I] as TDBTableForm;
      if DBTableForm.TableName = TableName then
      begin
        SetLength(DBTableForms, Length(DBTableForms) + 1);
        DBTableForms[High(DBTableForms)] := DBTableForm;
      end;
    end;
  end;
  for I := 0 to High(DBTableForms) do
    DBTableForms[I].Close;
  SetLength(DBTableForms, 0);
end;

procedure TDBInfoForm.CloseAllTable;
var
  I: Integer;
  DBTableForm: TDBTableForm;
  DBTableForms: array of TDBTableForm;
begin
  for I := 0 to MainForm.MDIChildCount - 1 do
  begin
    if MainForm.MDIChildren[I] is TDBTableForm then
    begin
      DBTableForm := MainForm.MDIChildren[I] as TDBTableForm;
      SetLength(DBTableForms, Length(DBTableForms) + 1);
      DBTableForms[High(DBTableForms)] := DBTableForm;
    end;
  end;
  for I := 0 to High(DBTableForms) do
    DBTableForms[I].Close;
  SetLength(DBTableForms, 0);
end;

procedure TDBInfoForm.ExportDBToText(FileName: string);
var
  TableIdx, RecordIdx, FieldIdx: Integer;
  Lines: TStrings;
  TinyTable: TTinyTable;
begin
  Lines := TStringList.Create;
  TinyTable := TTinyTable.Create(nil);
  MainForm.BeginProgress;
  try
    TinyTable.DatabaseName := FTinyDatabase.DatabaseName;
    for TableIdx := 0 to FTinyDatabase.TableDefs.Count - 1 do
    begin
      TinyTable.Close;
      TinyTable.TableName := FTinyDatabase.TableDefs[TableIdx].Name;
      TinyTable.Open;
      Lines.Add('Table Name: [' + TinyTable.TableName + ']');
      for RecordIdx := 0 to TinyTable.RecordCount - 1 do
      begin
        for FieldIdx := 0 to TinyTable.FieldDefs.Count - 1 do
        begin
          Lines.Add('[' +  TinyTable.Fields[FieldIdx].FieldName + ']:');
          Lines.Add(TinyTable.Fields[FieldIdx].AsString);
        end;
        Lines.Add('');
        TinyTable.Next;
        MainForm.DoProgress(Trunc((RecordIdx+1)/TinyTable.RecordCount*100));
      end;
      Lines.Add('--------------------------------');
    end;
    Lines.SaveToFile(FileName);
  finally
    Lines.Free;
    TinyTable.Close;
    TinyTable.Free;
    MainForm.EndProgress;
  end;
end;

procedure TDBInfoForm.LocateNodeOfTableName(TableName: string);
var
  I: Integer;
  NodeData: TNodeData;
begin
  for I := 0 to DBStructTreeView.Items.Count - 1 do
  begin
    if DBStructTreeView.Items[I].Data = nil then Continue;
    NodeData := GetNodeData(DBStructTreeView.Items[I]);
    if (NodeData.NodeType = ntTable) and (CompareText(NodeData.SelfName, TableName) = 0) then
    begin
      DBStructTreeView.Items[I].Selected := True;
      DBStructTreeView.Items[I].MakeVisible;
      FCurNode := DBStructTreeView.Items[I];
      Break;
    end;
  end;
end;

procedure TDBInfoForm.DeleteIndex(TableName, IndexName: string);
var
  R: Integer;
begin
  R := Application.MessageBox(PChar(AppLangMgr.Trans('Are you sure to delete?')), PChar(Application.Title), 36);
  if R = ID_YES then
  begin
    CloseFormOfTableName(TableName);
    FTinyDatabase.DeleteIndex(TableName, IndexName);
    FillDBStructToTreeView;
  end;
end;

procedure TDBInfoForm.RenameTable(OldTableName, NewTableName: string);
begin
  CloseFormOfTableName(OldTableName);
  FTinyDatabase.RenameTable(OldTableName, NewTableName);
end;

procedure TDBInfoForm.RenameField(TableName, OldFieldName, NewFieldName: string);
begin
  CloseFormOfTableName(TableName);
  FTinyDatabase.RenameField(TableName, OldFieldName, NewFieldName);
end;

procedure TDBInfoForm.RenameIndex(TableName, OldIndexName, NewIndexName: string);
begin
  CloseFormOfTableName(TableName);
  FTinyDatabase.RenameIndex(TableName, OldIndexName, NewIndexName);
end;

procedure TDBInfoForm.DBStructTreeViewMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Node, TempNode: TTreeNode;
  NodeData: TNodeData;
  Pos: TPoint;
  HitTest: THitTests;
  Empty: Boolean;
begin
  Pos := DBStructTreeView.ClientToScreen(Point(X, Y));
  HitTest := DBStructTreeView.GetHitTestInfoAt(X, Y);
  if (htOnIcon in HitTest) or (htOnLabel in HitTest)then
  begin
    TempNode := DBStructTreeView.GetNodeAt(X, Y);
    if TempNode <> nil then TempNode.Selected := True;
  end;

  Node := DBStructTreeView.Selected;
  if Node = nil then Exit;
  if Node.Data = nil then Exit;

  NodeData := GetNodeData(Node);
  FCurNodeData := NodeData;
  FCurNode := Node;

  if Button = mbRight then
  begin
    Empty := DBStructTreeView.Items.Count = 0;
    TableMenuOpenItem.Enabled := not Empty;
    TableMenuDeleteItem.Enabled := not Empty;
    TableMenuRenameItem.Enabled := not Empty;
    TableMenuEmptyItem.Enabled := not Empty;
    if Empty then TablePopupMenu.Popup(Pos.x, Pos.y);

    case NodeData.NodeType of
      ntTable:
        TablePopupMenu.Popup(Pos.x, Pos.y);
      ntField:
        FieldPopupMenu.Popup(Pos.x, Pos.y);
      ntIndex:
        IndexPopupMenu.Popup(Pos.x, Pos.y);
    end;
  end;
end;

procedure TDBInfoForm.DBStructTreeViewDblClick(Sender: TObject);
var
  Node: TTreeNode;
  NodeData: TNodeData;
begin
  Node := DBStructTreeView.Selected;
  if Node = nil then Exit;
  if Node.Data = nil then Exit;

  NodeData := GetNodeData(Node);
  case NodeData.NodeType of
    ntField:
      FieldMenuPropItemClick(nil);
    ntIndex:
      IndexMenuPropItemClick(nil);
  end;
end;

procedure TDBInfoForm.TableMenuOpenItemClick(Sender: TObject);
begin
  OpenTable(FCurNodeData.SelfName);
end;

procedure TDBInfoForm.TableMenuDeleteItemClick(Sender: TObject);
begin
  DeleteTable(FCurNodeData.SelfName);
end;

procedure TDBInfoForm.TableMenuRenameItemClick(Sender: TObject);
begin
  FCurNode.EditText;
end;

procedure TDBInfoForm.TableMenuRefreshItemClick(Sender: TObject);
begin
  FillDBStructToTreeView;
end;

procedure TDBInfoForm.IndexMenuPropItemClick(Sender: TObject);
var
  Value: TIndexPropFormData;
begin
  Value.TableName := FCurNodeData.TableName;
  Value.CurIndexName := FCurNodeData.SelfName;
  ShowIndexPropForm(Value);
end;

procedure TDBInfoForm.FieldMenuPropItemClick(Sender: TObject);
var
  Value: TFieldPropFormData;
begin
  Value.TableName := FCurNodeData.TableName;
  Value.CurFieldName := FCurNodeData.SelfName;
  ShowFieldPropForm(Value);
end;

procedure TDBInfoForm.TableMenuNewItemClick(Sender: TObject);
begin
  InvokeNewTableForm;
end;

procedure TDBInfoForm.TableMenuEmptyItemClick(Sender: TObject);
var
  Value: TTableListFormData;
  TableName: string;
begin
  if FCurNodeData.SelfName = '' then
  begin
    Value.TinyDatabase := FTinyDatabase;
    Value.TableName := '';
    if ShowTableListForm(Value) then
      TableName := Value.TableName
    else
      Exit;
  end else
    TableName := FCurNodeData.SelfName;

  EmptyTable(TableName);
end;

procedure TDBInfoForm.TableMenuDesignItemClick(Sender: TObject);
begin
  DesignTable(FCurNodeData.SelfName);
end;

procedure TDBInfoForm.IndexMenuDeleteItemClick(Sender: TObject);
begin
  DeleteIndex(FCurNodeData.TableName, FCurNodeData.SelfName);
end;

procedure TDBInfoForm.IndexMenuRenameItemClick(Sender: TObject);
begin
  FCurNode.EditText;
end;

procedure TDBInfoForm.FieldMenuRenameItemClick(Sender: TObject);
begin
  FCurNode.EditText;
end;

procedure TDBInfoForm.DBStructTreeViewEditing(Sender: TObject;
  Node: TTreeNode; var AllowEdit: Boolean);
begin
  if Node.Data = nil then AllowEdit := False;
end;

procedure TDBInfoForm.DBStructTreeViewEdited(Sender: TObject;
  Node: TTreeNode; var S: String);
var
  NodeData: TNodeData;
  TempNode: TTreeNode;
  I: Integer;
begin
  NodeData := GetNodeData(Node);
  case NodeData.NodeType of
    ntTable:
      begin
        RenameTable(NodeData.SelfName, S);
        MainForm.InitTableComboxBox;
        PNodeData(Node.Data)^.SelfName := S;
        for I := 0 to DBStructTreeView.Items.Count - 1 do
        begin
          TempNode := DBStructTreeView.Items[I];
          if TempNode.Data <> nil then
            if CompareText(GetNodeData(TempNode).TableName, NodeData.TableName) = 0 then
              PNodeData(TempNode.Data)^.TableName := S;
        end;
      end;
    ntField:
      begin
        RenameField(NodeData.TableName, NodeData.SelfName, S);
        PNodeData(Node.Data)^.SelfName := S;
      end;
    ntIndex:
      begin
        RenameIndex(NodeData.TableName, NodeData.SelfName, S);
        PNodeData(Node.Data)^.SelfName := S;
      end;
  end;
end;

end.
